c
c Copyright (c) 2001-2004 The Trustees of Indiana University.  
c                         All rights reserved.
c Copyright (c) 1998-2001 University of Notre Dame. 
c                         All rights reserved.
c Copyright (c) 1994-1998 The Ohio State University.  
c                         All rights reserved.
c 
c This file is part of the LAM/MPI software package.  For license
c information, see the LICENSE file in the top level directory of the
c LAM/MPI source distribution.
c 
c $HEADER$
c
c $Id$
c
c****************************************************************************
c
c MESSAGE PASSING INTERFACE TEST CASE SUITE
c
c Copyright IBM Corp. 1995
c
c       IBM Corp. hereby grants a non-exclusive license to use, copy,
c       modify, and distribute this software for any purpose and without
c       fee provided that the above copyright notice and the following
c       paragraphs appear in all copies.
c
c       IBM Corp. makes no representation that the test cases comprising
c       this suite are correct or are an accurate representation of any
c       standard.
c
c       In no event shall IBM be liable to any party for direct,
c       indirect, special incidental, or consequential damage arising
c       out of the use of this software even if IBM Corp. has been
c       advised of the possibility of such damage.
c
c       IBM CORP. SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT
c       NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
c       FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED
c       HEREUNDER IS ON AN "AS IS" BASIS AND IBM CORP. HAS NO OBLIGATION
c       TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
c       MODIFICATIONS.
c
c****************************************************************************
c
c       These test cases reflect an interpretation of the MPI Standard.
c       They are are, in most cases, unit tests of specific MPI
c       behaviors.  If a user of any test case from this set believes
c       that the MPI Standard requires behavior different than that
c       implied by the test case we would appreciate feedback.
c
c Comments may be sent to:
c    Richard Treumann
c    treumann@kgn.ibm.com
c
c****************************************************************************
c
c	Test the optional fortran integer types
c
      program optional

      include 'mpif.h'

      integer   rank
      integer   size
      integer   status(MPI_STATUS_SIZE)
      integer*4 int4
      integer*4 int4temp
      integer*4 int4array(3)
      integer min
      integer want_abort 
      integer tag
      integer ierror

c
c Initialize MPI.
c
      call MPI_INIT(ierror)
c
c Error check the number of processes.
c Determine my rank in the world group.
c The sender will be rank 0 and the receiver, rank 1.
c
      call MPI_COMM_SIZE(MPI_COMM_WORLD, size, ierror)
      call MPI_COMM_RANK(MPI_COMM_WORLD, rank, ierror)

c abort if the number of processes is less than 2 
      
      min = 2
      want_abort = 1

      call lamtest_check_size_f(min, want_abort)
c
c As rank 0, send a message to rank 1.
c

c      
c In the following lines, we send second element
c of an array across to the second process
c The first and third elements of the array are
c set to 0, the second element is set to some
c huge value in rank 0 and is sent across to 
c rank 2.  
c

c      
c on the receiving side we check the value of the
c received element and if it's not equal to the 
c expected value, then something is wrong, abort
c

      int4 = 1

c
c initialize array variables
c
      int4array(1) = 0
      int4array(2) = 1
      int4array(3) = 0

      do i = 1, 15
        int4 = int4 * 2
      end do  
      
      int4temp = z'123'   
      int4temp = z'1234567'
      
      int4 = int4 - int4temp
c
c Check if integer*4 is working properly       
c 
      tag = 202 
      if (rank .eq. 0) then
        int4array(2) = int4
        call MPI_SEND(int4array, 3, MPI_INTEGER4, 1, tag, 
     &             MPI_COMM_WORLD, ierror)
      endif 

      if (rank .eq. 1) then
        call MPI_Recv(int4array, 3, MPI_INTEGER4, 0, tag,
     &         MPI_COMM_WORLD, status, ierror)  
      
        if (int4array(2) .ne. int4) then
          call lamtest_error_f('MPI_INTEGER4 receive error')
        endif

      endif
c      
c End of program
c
      call MPI_FINALIZE(ierror)
      stop
      end
